"""
Agent loops for agent
"""

# Import the loops to register them
from . import (
    anthropic,
    composed_grounded,
    fara,
    gelato,
    gemini,
    generic_vlm,
    glm45v,
    gta1,
    holo,
    internvl,
    moondream3,
    omniparser,
    openai,
    opencua,
    uiins,
    uitars,
    uitars2,
)

__all__ = [
    "anthropic",
    "composed_grounded",
    "gelato",
    "gemini",
    "generic_vlm",
    "fara",
    "glm45v",
    "gta1",
    "holo",
    "internvl",
    "moondream3",
    "omniparser",
    "openai",
    "opencua",
    "uiins",
    "uitars",
    "uitars2",
]
